@@ -3,6 +3,7 @@
 #undef HAVE_LIBSM
 #undef ENABLE_NLS
 #undef HAVE_CATGETS
+#undef HAVE_DEFOPEN
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
@@ -10,17 +11,24 @@
 #undef HAVE_TCPWRAPPERS
 #undef HAVE_SHADOW
 #undef HAVE_CRYPT
-#undef HAVE_LIBXINERAMA
+#undef HAVE_XINERAMA
+#undef HAVE_XFREE_XINERAMA
+#undef HAVE_SOLARIS_XINERAMA
 #undef HAVE_LIBXDMCP
 #undef HAVE_SETENV
 #undef HAVE_UNSETENV
 #undef HAVE_CLEARENV
+#undef HAVE_SETRESUID
 #undef EXPANDED_DATADIR
 #undef EXPANDED_PIXMAPDIR
 #undef EXPANDED_BINDIR
 #undef EXPANDED_SBINDIR
+#undef EXPANDED_LIBEXECDIR
 #undef EXPANDED_GDMCONFIGDIR
 #undef EXPANDED_LOCALEDIR
 #undef EXPANDED_AUTHDIR
 #undef EXPANDED_SYSCONFDIR
 #undef EXPANDED_SESSDIR
+#undef GETTEXT_PACKAGE
+#undef HAVE_LOGINCAP
+#undef HAVE_SYS_SOCKIO_H
