@@ -50,16 +50,17 @@
 #define SERVER_ABORT 253	/* Server failed badly. Suspending display. */
 
 /* DO NOTE USE 1, that's used as error if x connection fails usually */
-/* Note that there is no reasons why these were a power of two, and note
+/* Note that there is no reason why these were a power of two, and note
  * that they have to fit in 256 */
 /* These are the exit codes */
 #define DISPLAY_REMANAGE 2	/* Restart display */
 #define DISPLAY_ABORT 4		/* Houston, we have a problem */
 #define DISPLAY_REBOOT 8	/* Rebewt */
 #define DISPLAY_HALT 16		/* Halt */
-#define DISPLAY_SUSPEND 17	/* Suspend */
+#define DISPLAY_SUSPEND 17	/* Suspend (don't use, use the interrupt) */
 #define DISPLAY_CHOSEN 20	/* successful chooser session,
 				   restart display */
+#define DISPLAY_RUN_CHOOSER 30	/* Run chooser */
 #define DISPLAY_XFAILED 64	/* X failed */
 #define DISPLAY_RESTARTGREETER 127 /* Restart greeter */
 #define DISPLAY_RESTARTGDM 128	/* Restart GDM */
@@ -77,7 +78,7 @@ enum {
 
 /* This will change if there are incompatible
  * protocol changes */
-#define GDM_GREETER_PROTOCOL_VERSION "2"
+#define GDM_GREETER_PROTOCOL_VERSION "3"
 
 #define GDM_MSG        'D'
 #define GDM_NOECHO     'U'
@@ -88,13 +89,14 @@ enum {
 #define GDM_SLANG      'R'
 #define GDM_RESET      'A'
 #define GDM_QUIT       'P'
-/* Well these aren't as nice as above, oh well */
+#if 0
+/* FIXME: Maybe just whack this */
 #define GDM_GNOMESESS  '?'
 #define GDM_SGNOMESESS '*'
+#endif
+/* Well these aren't as nice as above, oh well */
 #define GDM_STARTTIMER 's'
 #define GDM_STOPTIMER  'S'
-#define GDM_LOGIN      'L' /* this is the login prompt, much like PROMPT but
-			      different */
 #define GDM_SETLOGIN   'l' /* this just sets the login to be this, just for
 			      the greeters knowledge */
 #define GDM_DISABLE    '-' /* disable the login screen */
@@ -108,10 +110,13 @@ enum {
 #define GDM_NOFOCUS    'f' /* Don't focus the login window (optional) */
 #define GDM_FOCUS      'F' /* Allow focus on the login window again (optional) */
 #define GDM_SAVEDIE    '!' /* Save wm order and die (and set busy cursor) */
+#define GDM_QUERY_CAPSLOCK 'Q' /* Is capslock on? */
 
 /* Different login interruptions */
 #define GDM_INTERRUPT_TIMED_LOGIN 'T'
 #define GDM_INTERRUPT_CONFIGURE   'C'
+#define GDM_INTERRUPT_SUSPEND     'S'
+#define GDM_INTERRUPT_SELECT_USER 'U'
 
 /* The dreaded miscellaneous category */
 #define FIELD_SIZE 256
@@ -124,10 +129,11 @@ enum {
  * to nothing */
 #define GDM_KEY_AUTOMATICLOGIN_ENABLE "daemon/AutomaticLoginEnable=true"
 #define GDM_KEY_AUTOMATICLOGIN "daemon/AutomaticLogin="
-#define GDM_KEY_LOCALNOPASSWORDUSERS "daemon/LocalNoPasswordUsers="
 #define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=true"
 #define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin"
 #define GDM_KEY_REMOTEGREETER "daemon/RemoteGreeter=" EXPANDED_BINDIR "/gdmlogin"
+#define GDM_KEY_ADD_GTK_MODULES "daemon/AddGtkModules=false"
+#define GDM_KEY_GTK_MODULES_LIST "daemon/GtkModulesList=" "gail:atk-bridge"
 #define GDM_KEY_GROUP "daemon/Group=gdm"
 #define GDM_KEY_HALT "daemon/HaltCommand=/usr/bin/poweroff;/sbin/poweroff;/sbin/shutdown -h now;/usr/sbin/shutdown -h now"
 #define GDM_KEY_INITDIR "daemon/DisplayInitDir=" EXPANDED_SYSCONFDIR "/gdm/Init"
@@ -137,13 +143,19 @@ enum {
 #define GDM_KEY_PIDFILE "daemon/PidFile=/var/run/gdm.pid"
 #define GDM_KEY_POSTSESS "daemon/PostSessionScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PostSession/"
 #define GDM_KEY_PRESESS "daemon/PreSessionScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PreSession/"
+#define GDM_KEY_POSTLOGIN "daemon/PostLoginScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PreSession/"
 #define GDM_KEY_FAILSAFE_XSERVER "daemon/FailsafeXServer="
 #define GDM_KEY_XKEEPSCRASHING "daemon/XKeepsCrashing=" EXPANDED_SYSCONFDIR "/gdm/XKeepsCrashing"
 #define GDM_KEY_REBOOT "daemon/RebootCommand=/usr/bin/reboot;/sbin/reboot;/sbin/shutdown -r now;/usr/sbin/shutdown -r now"
 #define GDM_KEY_ROOTPATH "daemon/RootPath=/sbin:/usr/sbin:/bin:/usr/bin:/usr/bin/X11:/usr/local/bin:" EXPANDED_BINDIR
+#if 0
+/* FIXME: Maybe just whack this */
 #define GDM_KEY_GNOMEDEFAULTSESSION "daemon/GnomeDefaultSession=" EXPANDED_DATADIR "/gnome/default.session"
+#endif
 #define GDM_KEY_SERVAUTH "daemon/ServAuthDir=" EXPANDED_AUTHDIR
-#define GDM_KEY_SESSDIR "daemon/SessionDir=" EXPANDED_SYSCONFDIR "/gdm/Sessions/"
+#define GDM_KEY_SESSDIR "daemon/SessionDesktopDir=" EXPANDED_SYSCONFDIR "/dm/Sessions/"
+#define GDM_KEY_BASEXSESSION "daemon/BaseXsession=" EXPANDED_SYSCONFDIR "/gdm/Xsession"
+#define GDM_KEY_DEFAULTSESSION "daemon/DefaultSession=gnome.desktop"
 #define GDM_KEY_SUSPEND "daemon/SuspendCommand="
 
 #define GDM_KEY_UAUTHDIR "daemon/UserAuthDir="
@@ -175,6 +187,8 @@ enum {
 #define GDM_KEY_SERVER_CHOOSABLE "choosable=false"
 /* Login is handled by gdm, otherwise it's a remote server */
 #define GDM_KEY_SERVER_HANDLED "handled=true"
+/* Instead of the greeter run the chooser */
+#define GDM_KEY_SERVER_CHOOSER "chooser=false"
 
 #define GDM_KEY_ALLOWROOT "security/AllowRoot=true"
 #define GDM_KEY_ALLOWREMOTEROOT "security/AllowRemoteRoot=true"
@@ -183,6 +197,7 @@ enum {
 #define GDM_KEY_SESSIONMAXFILE "security/SessionMaxFile=524288"
 #define GDM_KEY_RELAXPERM "security/RelaxPermissions=0"
 #define GDM_KEY_RETRYDELAY "security/RetryDelay=3"
+#define GDM_KEY_DISALLOWTCP "security/DisallowTCP=true"
 
 #define GDM_KEY_XDMCP "xdmcp/Enable=false"
 #define GDM_KEY_MAXPEND "xdmcp/MaxPending=4"
@@ -193,7 +208,7 @@ enum {
 #define GDM_KEY_INDIRECT "xdmcp/HonorIndirect=true"
 #define GDM_KEY_MAXINDIR "xdmcp/MaxPendingIndirect=4"
 #define GDM_KEY_MAXINDWAIT "xdmcp/MaxWaitIndirect=30"
-#define GDM_KEY_PINGINTERVAL "xdmcp/PingInterval=1"
+#define GDM_KEY_PINGINTERVAL "xdmcp/PingIntervalSeconds=15"
 #define GDM_KEY_WILLING "xdmcp/Willing=" EXPANDED_SYSCONFDIR "/gdm/Xwilling"
 
 #define GDM_KEY_GTKRC "gui/GtkRC=" EXPANDED_DATADIR "/themes/Default/gtk/gtkrc"
@@ -212,8 +227,10 @@ enum {
 #define GDM_KEY_SYSMENU "greeter/SystemMenu=true"
 #define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_GDMCONFIGDIR "/gdmsetup --disable-sound --disable-crash-dialog"
 #define GDM_KEY_CONFIG_AVAILABLE "greeter/ConfigAvailable=true"
+#define GDM_KEY_CHOOSER_BUTTON "greeter/ChooserButton=true"
 #define GDM_KEY_TITLE_BAR "greeter/TitleBar=true"
-#define GDM_KEY_WELCOME "greeter/Welcome=Welcome to %n"
+#define GDM_KEY_WELCOME "greeter/Welcome=Welcome"
+#define GDM_KEY_REMOTEWELCOME "greeter/RemoteWelcome=Welcome to %n"
 #define GDM_KEY_XINERAMASCREEN "greeter/XineramaScreen=0"
 #define GDM_KEY_BACKGROUNDPROG "greeter/BackgroundProgram="
 #define GDM_KEY_RUNBACKGROUNDPROGALWAYS "greeter/RunBackgroundProgramAlways=false"
@@ -231,24 +248,31 @@ enum {
 #define GDM_KEY_GRAPHICAL_THEME "greeter/GraphicalTheme=circles"
 #define GDM_KEY_GRAPHICAL_THEME_DIR "greeter/GraphicalThemeDir=" EXPANDED_DATADIR "/gdm/themes/"
 
-#define GDM_KEY_SCAN "chooser/ScanTime=3"
+#define GDM_KEY_SCAN "chooser/ScanTime=4"
 #define GDM_KEY_HOST "chooser/DefaultHostImg=" EXPANDED_PIXMAPDIR "/nohost.png"
 #define GDM_KEY_HOSTDIR "chooser/HostImageDir=" EXPANDED_DATADIR "/hosts/"
 #define GDM_KEY_HOSTS "chooser/Hosts="
 #define GDM_KEY_BROADCAST "chooser/Broadcast=true"
+#define GDM_KEY_ALLOWADD "chooser/AllowAdd=true"
 
 #define GDM_KEY_DEBUG "debug/Enable=false"
 
 #define GDM_KEY_SERVERS "servers"
 
+#if 0
+/* FIXME: Maybe just whack this */
 #define GDM_KEY_SHOW_GNOME_CHOOSER "greeter/ShowGnomeChooserSession=true"
+#endif
 #define GDM_KEY_SHOW_GNOME_FAILSAFE "greeter/ShowGnomeFailsafeSession=true"
 #define GDM_KEY_SHOW_XTERM_FAILSAFE "greeter/ShowXtermFailsafeSession=true"
 #define GDM_KEY_SHOW_LAST_SESSION "greeter/ShowLastSession=true"
 
 #define GDM_SESSION_FAILSAFE_GNOME "GDM_Failsafe.GNOME"
 #define GDM_SESSION_FAILSAFE_XTERM "GDM_Failsafe.XTERM"
+#if 0
+/* FIXME: Maybe just whack this */
 #define GDM_SESSION_GNOME_CHOOSER "Gnome Chooser"
+#endif
 
 #define GDM_STANDARD "Standard"
 
@@ -274,7 +298,9 @@ struct _GdmDisplay {
     gboolean authfb;
     gchar *command;
     gboolean failsafe_xserver;
-    gboolean use_chooser;
+    gboolean use_chooser; /* run chooser instead of greeter */
+    gchar *chosen_hostname; /* locally chosen hostname if not NULL,
+			       "-query chosen_hostname" is appened to server command line */
     guint indirect_id;
     gchar *cookie;
     gchar *bcookie;
@@ -302,6 +328,7 @@ struct _GdmDisplay {
     gboolean console;
 
     time_t last_start_time;
+    time_t last_loop_start_time;
     gint retry_count;
 
     int sleep_before_run;
@@ -345,6 +372,7 @@ struct _GdmXServer {
 	char *command;
 	gboolean flexible;
 	gboolean choosable; /* not implemented yet */
+	gboolean chooser; /* instead of greeter, run chooser */
 	gboolean handled;
 };
 
@@ -408,6 +436,7 @@ void		gdm_final_cleanup	(void);
  * the slave will be sent a SIGUSR2 */
 /* The fifo protocol, used only by gdm internally */
 #define GDM_SOP_CHOSEN       "CHOSEN" /* <indirect id> <ip addr> */
+#define GDM_SOP_CHOSEN_LOCAL "CHOSEN_LOCAL" /* <slave pid> <hostname> */
 #define GDM_SOP_XPID         "XPID" /* <slave pid> <xpid> */
 #define GDM_SOP_SESSPID      "SESSPID" /* <slave pid> <sesspid> */
 #define GDM_SOP_GREETPID     "GREETPID" /* <slave pid> <greetpid> */
@@ -442,6 +471,8 @@ void		gdm_final_cleanup	(void);
  * by asking the user if they want to do that.  Note that this will not
  * kill any logged in sessions. */
 #define GDM_SOP_SOFT_RESTART_SERVERS "SOFT_RESTART_SERVERS"  /* no arguments */
+/* Suspend the machine if it is even allowed */
+#define GDM_SOP_SUSPEND_MACHINE "SUSPEND_MACHINE"  /* no arguments */
 
 /* Notification protocol */
 /* keys */
@@ -450,11 +481,13 @@ void		gdm_final_cleanup	(void);
 #define GDM_NOTIFY_ALLOWREMOTEAUTOLOGIN "AllowRemoteAutoLogin" /* <true/false as int> */
 #define GDM_NOTIFY_SYSMENU "SystemMenu" /* <true/false as int> */
 #define GDM_NOTIFY_CONFIG_AVAILABLE "ConfigAvailable" /* <true/false as int> */
+#define GDM_NOTIFY_CHOOSER_BUTTON "ChooserButton" /* <true/false as int> */
 #define GDM_NOTIFY_RETRYDELAY "RetryDelay" /* <seconds> */
 #define GDM_NOTIFY_GREETER "Greeter" /* <greeter binary> */
 #define GDM_NOTIFY_REMOTEGREETER "RemoteGreeter" /* <greeter binary> */
 #define GDM_NOTIFY_TIMED_LOGIN "TimedLogin" /* <login> */
 #define GDM_NOTIFY_TIMED_LOGIN_DELAY "TimedLoginDelay" /* <seconds> */
+#define GDM_NOTIFY_DISALLOWTCP "DisallowTCP" /* <true/false as int> */
 
 /* commands */
 #define GDM_NOTIFY_DIRTY_SERVERS "DIRTY_SERVERS"
@@ -477,9 +510,6 @@ void		gdm_final_cleanup	(void);
  * is the gdm version and not a "protocol" revision, so you can't check
  * against a single version but check if the version is higher then some
  * value.
- *
- * You can only send a few commands at a time, so if you keep getting error
- * 200 try opening a new socket for every command you send.
  */
 /* The user protocol, using /tmp/.gdm_socket */
 
@@ -489,9 +519,6 @@ void		gdm_final_cleanup	(void);
  * Arguments:  None
  * Answers:
  *   GDM <gdm version>
- *   ERROR <err number> <english error description>
- *      200 = Too many messages
- *      999 = Unknown error
  */
 #define GDM_SUP_AUTH_LOCAL "AUTH_LOCAL" /* <xauth cookie> */
 /* AUTH_LOCAL: Setup this connection as authenticated for FLEXI_SERVER
@@ -508,7 +535,6 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      100 = Not authenticated
- *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_FLEXI_XSERVER "FLEXI_XSERVER" /* <xserver type> */
@@ -527,7 +553,6 @@ void		gdm_final_cleanup	(void);
  *      4 = X too busy
  *      6 = No server binary
  *      100 = Not authenticated
- *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_FLEXI_XNEST  "FLEXI_XNEST" /* <display> <uid> <xauth cookie> <xauth file> */
@@ -556,7 +581,6 @@ void		gdm_final_cleanup	(void);
  *      5 = Xnest can't connect
  *      6 = No server binary
  *      100 = Not authenticated
- *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_CONSOLE_SERVERS  "CONSOLE_SERVERS" /* None */
@@ -574,14 +598,9 @@ void		gdm_final_cleanup	(void);
  *   for example).  If the display is an xnest display and is a console one
  *   (that is, it is an xnest inside another console display) it is listed
  *   and instead of vt, it lists the parent display in standard form.
- *
- *   ERROR <err number> <english error description>
- *      1 = Not implemented
- *      200 = Too many messages
- *      999 = Unknown error
  */
 #define GDM_SUP_UPDATE_CONFIG "UPDATE_CONFIG" /* <key> */
-/* AUTH_LOCAL: Tell the daemon to update config of some key.  Any user
+/* UPDATE_CONFIG: Tell the daemon to update config of some key.  Any user
  *             can really request that values are re-read but the daemon
  *             caches the last date of the config file so a user can't
  *             actually change any values unless they can write the
@@ -590,6 +609,7 @@ void		gdm_final_cleanup	(void);
  *   		 security/AllowRemoteRoot (2.3.90.2)
  *   		 security/AllowRemoteAutoLogin (2.3.90.2)
  *   		 security/RetryDelay (2.3.90.2)
+ *   		 security/DisallowTCP (2.4.2.0)
  *   		 daemon/Greeter (2.3.90.2)
  *   		 daemon/RemoteGreeter (2.3.90.2)
  *   		 xdmcp/Enable (2.3.90.2)
@@ -602,12 +622,13 @@ void		gdm_final_cleanup	(void);
  *			xdmcp/HonorIndirect
  *			xdmcp/MaxPendingIndirect
  *			xdmcp/MaxWaitIndirect
- *			xdmcp/PingInterval (only affects new connections)
+ *			xdmcp/PingIntervalSeconds (only affects new connections)
  *   		 xdmcp/TimedLogin (2.3.90.3)
  *   		 xdmcp/TimedLoginEnable (2.3.90.3)
  *   		 xdmcp/TimedLoginDelay (2.3.90.3)
  *   		 greeter/SystemMenu (2.3.90.3)
  *   		 greeter/ConfigAvailable (2.3.90.3)
+ *   		 greeter/ChooserButton (2.4.2.0)
  * Supported since: 2.3.90.2
  * Arguments:  <key>
  *   <key> is just the base part of the key such as "security/AllowRemoteRoot"
@@ -616,7 +637,6 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      50 = Unsupported key
- *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_GREETERPIDS  "GREETERPIDS" /* None */
@@ -626,10 +646,6 @@ void		gdm_final_cleanup	(void);
  * Arguments:  None
  * Answers:
  *   OK <pid>;<pid>;...
- *   ERROR <err number> <english error description>
- *      0 = Not implemented
- *      200 = Too many messages
- *      999 = Unknown error
  */
 #define GDM_SUP_CLOSE        "CLOSE" /* no arguments */
 /* CLOSE Answers: None
@@ -642,12 +658,6 @@ enum {
 					  * from a local display we started */
 };
 
-#define IGNORE_EINTR(expr) \
-	do {		\
-		errno = 0;	\
-		expr;		\
-	} while (errno == EINTR);
-
 #endif /* GDM_H */
 
 /* EOF */
