@@ -42,6 +42,35 @@ extern gboolean GdmAllowRoot;
 extern gboolean GdmAllowRemoteRoot;
 extern gint GdmRetryDelay;
 
+static void
+print_cant_auth_errbox (void)
+{
+	gboolean is_capslock = FALSE;
+	const char *basemsg;
+	char *msg;
+	char *ret;
+
+	ret = gdm_slave_greeter_ctl (GDM_QUERY_CAPSLOCK, "");
+	if ( ! ve_string_empty (ret))
+		is_capslock = TRUE;
+	g_free (ret);
+
+	basemsg = _("\nIncorrect username or password.  "
+		    "Letters must be typed in the correct "
+		    "case.");
+	if (is_capslock) {
+		msg = g_strconcat (basemsg, "  ",
+				   _("Please make sure the "
+				     "Caps Lock key is not "
+				     "enabled."),
+				   NULL);
+	} else {
+		msg = g_strdup (basemsg);
+	}
+	gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, msg);
+	g_free (msg);
+}
+
 /**
  * gdm_verify_user:
  * @username: Name of user or NULL if we should ask
@@ -69,7 +98,7 @@ gdm_verify_user (GdmDisplay *d,
     /* Ask for the user's login */
     if (username == NULL) {
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, _("Please enter your username"));
-	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Username:"));
+	    login = gdm_slave_greeter_ctl (GDM_PROMPT, _("Username:"));
 	    if (login == NULL ||
 		gdm_slave_greeter_check_interruption ()) {
 		    if (local)
@@ -81,6 +110,7 @@ gdm_verify_user (GdmDisplay *d,
     } else {
 	    login = g_strdup (username);
     }
+    gdm_slave_greeter_ctl_no_ret (GDM_SETLOGIN, login);
 
     pwent = getpwnam (login);
         
@@ -88,12 +118,11 @@ gdm_verify_user (GdmDisplay *d,
     
     /* Request the user's password */
     if (pwent != NULL &&
-        (ve_string_empty (ppasswd) ||
-	 (local && gdm_is_a_no_password_user (login)))) {
+        ve_string_empty (ppasswd)) {
 	    /* eeek a passwordless account */
 	    passwd = g_strdup ("");
     } else {
-	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
+	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password:"));
 	    if (passwd == NULL)
 		    passwd = g_strdup ("");
 	    if (gdm_slave_greeter_check_interruption ()) {
@@ -110,18 +139,11 @@ gdm_verify_user (GdmDisplay *d,
 	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 
     if (pwent == NULL) {
-	    gdm_sleep_no_signal (GdmRetryDelay);
-	    gdm_error (_("Couldn't authenticate user"));
-	    /* FIXME: Hmm, how are we sure that the login is username
-	     * and password.  That is the most common case but not
-	     * necessarily true, this message needs to be changed
-	     * to allow for such cases */
-	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password.  "
-		       "Letters must be typed in the correct case.  "  
-		       "Please be sure the Caps Lock key is not enabled"));
-	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-	    g_free (auth_errmsg);
+	    sleep (GdmRetryDelay);
+	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
+
+	    print_cant_auth_errbox ();
+
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -131,17 +153,11 @@ gdm_verify_user (GdmDisplay *d,
     /* Check whether password is valid */
     if (ppasswd == NULL || (ppasswd[0] != '\0' &&
 			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
-	    gdm_sleep_no_signal (GdmRetryDelay);
-	    /* FIXME: Hmm, how are we sure that the login is username
-	     * and password.  That is the most common case but not
-	     * necessarily true, this message needs to be changed
-	     * to allow for such cases */
-	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password.  "
-		       "Letters must be typed in the correct case.  "  
-		       "Please be sure the Caps Lock key is not enabled"));
-	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-	    g_free (auth_errmsg);
+	    sleep (GdmRetryDelay);
+	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
+
+	    print_cant_auth_errbox ();
+
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -209,10 +225,13 @@ gdm_verify_user (GdmDisplay *d,
 
 gboolean
 gdm_verify_setup_user (GdmDisplay *d,
-		       const gchar *login, const gchar *display) 
+		       const gchar *login, const gchar *display,
+		       char **new_login) 
 {
 	struct passwd *pwent;
 
+	*new_login = NULL;
+
 	pwent = getpwnam (login);
 	if (pwent == NULL) {
 		gdm_error (_("Cannot get passwd structure for %s"), login);
