@@ -27,6 +27,10 @@
 #include <security/pam_appl.h>
 #include <pwd.h>
 
+#ifdef HAVE_DEFOPEN
+#include <deflt.h>
+#endif
+
 #include <vicious.h>
 
 #include "gdm.h"
@@ -48,8 +52,62 @@ static pam_handle_t *pamh = NULL;
 
 static GdmDisplay *cur_gdm_disp = NULL;
 
-static gboolean opened_session = FALSE;
-static gboolean did_setcred = FALSE;
+/* this is a hack */
+static char *tmp_PAM_USER = NULL;
+
+/* this is another hack */
+static gboolean did_we_ask_for_password = FALSE;
+
+static char *selected_user = NULL;
+
+void
+gdm_verify_select_user (const char *user)
+{
+	g_free (selected_user);
+	if (ve_string_empty (user))
+		selected_user = NULL;
+	else
+		selected_user = g_strdup (user);
+}
+
+static const char *
+perhaps_translate_message (const char *msg)
+{
+	char *s;
+	const char *ret;
+	static GHashTable *hash = NULL;
+	if (hash == NULL) {
+		/* Here we come with some fairly standard messages so that
+		   we have as much as possible translated.  Should really be
+		   translated in pam I suppose */
+		hash = g_hash_table_new (g_str_hash, g_str_equal);
+		/* login: is whacked always translate to Username: */
+		g_hash_table_insert (hash, "login:", _("Username:"));
+		g_hash_table_insert (hash, "Username:", _("Username:"));
+		g_hash_table_insert (hash, "Password:", _("Password:"));
+		g_hash_table_insert (hash, "You are required to change your password immediately (password aged)", _("You are required to change your password immediately (password aged)"));
+		g_hash_table_insert (hash, "You are required to change your password immediately (root enforced)", _("You are required to change your password immediately (root enforced)"));
+		g_hash_table_insert (hash, "Your account has expired; please contact your system administrator", _("Your account has expired; please contact your system administrator"));
+		g_hash_table_insert (hash, "No password supplied", _("No password supplied"));
+		g_hash_table_insert (hash, "Password unchanged", _("Password unchanged"));
+		g_hash_table_insert (hash, "Can not get username", _("Can not get username"));
+		g_hash_table_insert (hash, "Retype new UNIX password:", _("Retype new UNIX password:"));
+		g_hash_table_insert (hash, "Enter new UNIX password:", _("Enter new UNIX password:"));
+		g_hash_table_insert (hash, "(current) UNIX password:", _("(current) UNIX password:"));
+		g_hash_table_insert (hash, "Error while changing NIS password.", _("Error while changing NIS password."));
+		g_hash_table_insert (hash, "You must choose a longer password", _("You must choose a longer password"));
+		g_hash_table_insert (hash, "Password has been already used. Choose another.", _("Password has been already used. Choose another."));
+		g_hash_table_insert (hash, "You must wait longer to change your password", _("You must wait longer to change your password"));
+		g_hash_table_insert (hash, "Sorry, passwords do not match", _("Sorry, passwords do not match"));
+	}
+	s = g_strstrip (g_strdup (msg));
+	ret = g_hash_table_lookup (hash, s);
+	g_free (s);
+	if (ret != NULL)
+		return ret;
+	else
+		return msg;
+}
 
 /* Internal PAM conversation function. Interfaces between the PAM
  * authentication system and the actual greeter program */
@@ -60,8 +118,10 @@ gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
 		     void *appdata_ptr)
 {
     int replies = 0;
+    int i;
     char *s;
     struct pam_response *reply = NULL;
+    const char *login;
     
     reply = malloc (sizeof (struct pam_response) * num_msg);
     
@@ -69,35 +129,83 @@ gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
 	return PAM_CONV_ERR;
 
     memset (reply, 0, sizeof (struct pam_response) * num_msg);
+
+    /* Here we set the login if it wasn't already set,
+     * this is kind of anal, but this way we guarantee that
+     * the greeter always is up to date on the login */
+    if (pam_get_item (pamh, PAM_USER, (const void **)&login) == PAM_SUCCESS ||
+	tmp_PAM_USER != NULL) {
+	    /* FIXME: this is a HACK HACK HACK, for some reason needed */
+	    if (tmp_PAM_USER != NULL)
+		    login = tmp_PAM_USER;
+	    gdm_slave_greeter_ctl_no_ret (GDM_SETLOGIN, login);
+    }
     
     for (replies = 0; replies < num_msg; replies++) {
+	gboolean islogin = FALSE;
+	const char *m = msg[replies]->msg;
+	m = perhaps_translate_message (m);
 	
 	switch (msg[replies]->msg_style) {
 	    
+	/* PAM requested textual input with echo on */
 	case PAM_PROMPT_ECHO_ON:
-	    /* PAM requested textual input with echo on */
-	    s = gdm_slave_greeter_ctl (GDM_PROMPT, msg[replies]->msg);
+ 	    if (strcmp(m, _("Username:")) == 0) {
+		    /* this is an evil hack, but really there is no way we'll
+		       know this is a username prompt.  However we SHOULD NOT
+		       rely on this working.  The pam modules can set their
+		       prompt to whatever they wish to */
+		    gdm_slave_greeter_ctl_no_ret
+			    (GDM_MSG, _("Please enter your username"));
+		    s = gdm_slave_greeter_ctl (GDM_PROMPT, m);
+		    /* this will clear the message */
+		    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
+		    islogin = TRUE;
+	    } else {
+		    s = gdm_slave_greeter_ctl (GDM_PROMPT, m);
+	    }
 
 	    if (gdm_slave_greeter_check_interruption ()) {
 		    g_free (s);
+		    for (i = 0; i < replies; i++)
+			    if (reply[replies].resp != NULL)
+				    free (reply[replies].resp);
 		    free (reply);
 		    return PAM_CONV_ERR;
 	    }
 
+	    if (islogin) {
+		    /* note that we should NOT rely on this for anything
+		       that is REALLY needed.  The reason this is here
+		       is for the face browser to work correctly.  The
+		       best way would really be to ask all questions at
+		       once, but that would 1) need some protocol rethinking,
+		       2) the gdmgreeter program is really not equipped
+		       for this, would need theme format change */
+		    /* Second problem is that pam will not actually set
+		       PAM_USER itself after it asks for it.  This is
+		       pretty evil, evil, evil, evil, but somewhat necessary
+		       then. */
+		    /* FIXME: this is a HACK HACK HACK */
+		    g_free (tmp_PAM_USER);
+		    tmp_PAM_USER = g_strdup (s);
+	    }
+
 	    reply[replies].resp_retcode = PAM_SUCCESS;
 	    reply[replies].resp = strdup (ve_sure_string (s));
 	    g_free (s);
 	    break;
 	    
 	case PAM_PROMPT_ECHO_OFF:
+	    if (strcmp (m, _("Password:")) == 0)
+		    did_we_ask_for_password = TRUE;
 	    /* PAM requested textual input with echo off */
-	    if (strcmp (msg[replies]->msg, "Password: ") == 0)
-		    /* hack to make sure we translate "Password: " */
-		    s = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
-	    else
-		    s = gdm_slave_greeter_ctl (GDM_NOECHO, msg[replies]->msg);
+	    s = gdm_slave_greeter_ctl (GDM_NOECHO, m);
 	    if (gdm_slave_greeter_check_interruption ()) {
 		    g_free (s);
+		    for (i = 0; i < replies; i++)
+			    if (reply[replies].resp != NULL)
+				    free (reply[replies].resp);
 		    free (reply);
 		    return PAM_CONV_ERR;
 	    }
@@ -108,19 +216,22 @@ gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
 	    
 	case PAM_ERROR_MSG:
 	    /* PAM sent a message that should displayed to the user */
-	    gdm_slave_greeter_ctl (GDM_ERRDLG, msg[replies]->msg);
+	    gdm_slave_greeter_ctl (GDM_ERRDLG, m);
 	    reply[replies].resp_retcode = PAM_SUCCESS;
 	    reply[replies].resp = NULL;
 	    break;
 	case PAM_TEXT_INFO:
 	    /* PAM sent a message that should displayed to the user */
-	    gdm_slave_greeter_ctl (GDM_MSG, msg[replies]->msg);
+	    gdm_slave_greeter_ctl (GDM_MSG, m);
 	    reply[replies].resp_retcode = PAM_SUCCESS;
 	    reply[replies].resp = NULL;
 	    break;
 	    
 	default:
 	    /* PAM has been smoking serious crack */
+	    for (i = 0; i < replies; i++)
+		    if (reply[replies].resp != NULL)
+			    free (reply[replies].resp);
 	    free (reply);
 	    return PAM_CONV_ERR;
 	}
@@ -146,6 +257,7 @@ gdm_verify_standalone_pam_conv (int num_msg, const struct pam_message **msg,
 				void *appdata_ptr)
 {
 	int replies = 0;
+	int i;
 	char *s, *text;
 	struct pam_response *reply = NULL;
 
@@ -157,15 +269,17 @@ gdm_verify_standalone_pam_conv (int num_msg, const struct pam_message **msg,
 	memset (reply, 0, sizeof (struct pam_response) * num_msg);
 
 	for (replies = 0; replies < num_msg; replies++) {
+		const char *m = msg[replies]->msg;
+		m = perhaps_translate_message (m);
 		switch (msg[replies]->msg_style) {
 
 		case PAM_PROMPT_ECHO_ON:
 			if (extra_standalone_message != NULL)
 				text = g_strdup_printf
 					("%s\n%s", extra_standalone_message,
-					 msg[replies]->msg);
+					 m);
 			else
-				text = g_strdup (msg[replies]->msg);
+				text = g_strdup (m);
 
 			/* PAM requested textual input with echo on */
 			s = gdm_failsafe_question (cur_gdm_disp, text,
@@ -181,9 +295,9 @@ gdm_verify_standalone_pam_conv (int num_msg, const struct pam_message **msg,
 			if (extra_standalone_message != NULL)
 				text = g_strdup_printf
 					("%s\n%s", extra_standalone_message,
-					 msg[replies]->msg);
+					 m);
 			else
-				text = g_strdup (msg[replies]->msg);
+				text = g_strdup (m);
 
 			/* PAM requested textual input with echo off */
 			s = gdm_failsafe_question (cur_gdm_disp, text,
@@ -200,7 +314,7 @@ gdm_verify_standalone_pam_conv (int num_msg, const struct pam_message **msg,
 			/* PAM sent a message that should displayed to the user */
 			gdm_error_box (cur_gdm_disp,
 				       GTK_MESSAGE_ERROR,
-				       msg[replies]->msg);
+				       m);
 			reply[replies].resp_retcode = PAM_SUCCESS;
 			reply[replies].resp = NULL;
 			break;
@@ -209,13 +323,16 @@ gdm_verify_standalone_pam_conv (int num_msg, const struct pam_message **msg,
 			/* PAM sent a message that should displayed to the user */
 			gdm_error_box (cur_gdm_disp,
 				       GTK_MESSAGE_INFO,
-				       msg[replies]->msg);
+				       m);
 			reply[replies].resp_retcode = PAM_SUCCESS;
 			reply[replies].resp = NULL;
 			break;
 
 		default:
 			/* PAM has been smoking serious crack */
+			for (i = 0; i < replies; i++)
+				if (reply[replies].resp != NULL)
+					free (reply[replies].resp);
 			free (reply);
 			return PAM_CONV_ERR;
 		}
@@ -240,10 +357,8 @@ create_pamh (GdmDisplay *d,
 	     const char *display,
 	     int *pamerr)
 {
-	if (login == NULL ||
-	    display == NULL) {
-		gdm_error (_("Cannot setup pam handle with null login "
-			     "and/or display"));
+	if (display == NULL) {
+		gdm_error (_("Cannot setup pam handle with null display"));
 		return FALSE;
 	}
 
@@ -252,8 +367,6 @@ create_pamh (GdmDisplay *d,
 		pam_end (pamh, PAM_SUCCESS);
 	}
 	pamh = NULL;
-	opened_session = FALSE;
-	did_setcred = FALSE;
 
 	/* Initialize a PAM session for the user */
 	if ((*pamerr = pam_start (service, login, conv, &pamh)) != PAM_SUCCESS) {
@@ -269,6 +382,17 @@ create_pamh (GdmDisplay *d,
 		return FALSE;
 	}
 
+#if 0
+	/* Apparently we should not set RUSER, docs are unclear but the sun
+           guys are saying don't do it */
+	/* gdm is requesting the login */
+	if ((*pamerr = pam_set_item (pamh, PAM_RUSER, GdmUser)) != PAM_SUCCESS) {
+		if (gdm_slave_should_complain ())
+			gdm_error (_("Can't set PAM_RUSER=%s"), GdmUser);
+		return FALSE;
+	}
+#endif
+
 	if ( ! d->console) {
 		/* Only set RHOST if host is remote */
 		/* From the host of the display */
@@ -304,13 +428,12 @@ gdm_verify_user (GdmDisplay *d,
 		 gboolean local) 
 {
     gint pamerr = 0;
-    char *login;
-    const char *service = "gdm";
+    char *login = NULL;
     struct passwd *pwent;
     gboolean error_msg_given = FALSE;
     gboolean credentials_set = FALSE;
     gboolean started_timer = FALSE;
-    char *auth_errmsg;
+    int null_tok = 0;
 
     /* start the timer for timed logins */
     if ( ! ve_string_empty (GdmTimedLogin) &&
@@ -319,58 +442,95 @@ gdm_verify_user (GdmDisplay *d,
 	    started_timer = TRUE;
     }
 
-    if (username == NULL) {
-	    /* Ask gdmgreeter for the user's login. Just for good measure */
-	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, _("Please enter your username"));
-	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Username:"));
-	    if (login == NULL ||
-		gdm_slave_greeter_check_interruption ()) {
-		    if (started_timer)
-			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
-		    g_free (login);
-		    return NULL;
-	    }
-	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
-    } else {
+    if (username != NULL) {
 	    login = g_strdup (username);
-    }
-
-    if (local &&
-	gdm_is_a_no_password_user (login)) {
-	    service = "gdm-autologin";
-    } else {
-	    service = "gdm";
+	    gdm_slave_greeter_ctl_no_ret (GDM_SETLOGIN, login);
     }
 
     cur_gdm_disp = d;
 
     /* Initialize a PAM session for the user */
-    if ( ! create_pamh (d, service, login, &pamc, display, &pamerr)) {
+    if ( ! create_pamh (d, "gdm", username, &pamc, display, &pamerr)) {
 	    if (started_timer)
 		    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 	    goto pamerr;
     }
+
+    pam_set_item (pamh, PAM_USER_PROMPT, _("Username:"));
+
+    /* A Solaris thing: */
+#ifdef HAVE_DEFOPEN
+    if (defopen(DEFLT"/login") == 0) {
+	    char *passreq;
+	    int def_flgs;
+
+	    def_flgs = defcntl(DC_GETFLAGS, 0);
+	    TURNOFF(def_flgs, DC_CASE);
+	    (void) defcntl (DC_SETFLAGS, def_flgs);	 /* ignore case */
+
+	    if (((passreq = defread("PASSREQ=")) != NULL) &&
+		g_ascii_strcasecmp (passreq, "YES") == 0)
+		    null_tok |= PAM_DISALLOW_NULL_AUTHTOK;
+
+	    (void) defopen(NULL); /* close defaults file  */
+    }
+#endif
 	    
 #ifdef PAM_FAIL_DELAY
     pam_fail_delay (pamh, GdmRetryDelay * 1000);
 #endif /* PAM_FAIL_DELAY */
 
+authenticate_again:
+    /* hack */
+    g_free (tmp_PAM_USER);
+    tmp_PAM_USER = NULL;
+
+    did_we_ask_for_password = FALSE;
+
+    gdm_verify_select_user (NULL);
     /* Start authentication session */
-    if ((pamerr = pam_authenticate (pamh, 0)) != PAM_SUCCESS) {
-#ifndef PAM_FAIL_DELAY
-	    gdm_sleep_no_signal (GdmRetryDelay);
-#endif /* PAM_FAIL_DELAY */
+    if ((pamerr = pam_authenticate (pamh, null_tok)) != PAM_SUCCESS) {
+	    if ( ! ve_string_empty (selected_user)) {
+		    /* FIXME: what about errors */
+		    pam_set_item (pamh, PAM_USER, selected_user);
+		    /* Note that the GDM_SETUSER will be sent in the
+		       authenticate conversation.  This is a more robust
+		       solution. */
+		    goto authenticate_again;
+	    }
 	    if (started_timer)
 		    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
-	    if (gdm_slave_should_complain ())
+	    if (gdm_slave_should_complain ()) {
+#ifndef PAM_FAIL_DELAY
+		    sleep (GdmRetryDelay);
+#endif /* PAM_FAIL_DELAY */
 		    gdm_error (_("Couldn't authenticate user"));
+	    }
 	    goto pamerr;
     }
 
     /* stop the timer for timed logins */
     if (started_timer)
 	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
+
+    g_free (login);
+    login = NULL;
     
+    if ((pamerr = pam_get_item (pamh, PAM_USER, (const void **)&login))
+	!= PAM_SUCCESS) {
+	    login = NULL;
+	    /* is not really an auth problem, but it will
+	       pretty much look as such, it shouldn't really
+	       happen */
+	    if (gdm_slave_should_complain ())
+		    gdm_error (_("Couldn't authenticate user"));
+	    goto pamerr;
+    }
+    login = g_strdup (login);
+    /* kind of anal, the greeter likely already knows, but it could have
+       been changed */
+    gdm_slave_greeter_ctl_no_ret (GDM_SETLOGIN, login);
+
     pwent = getpwnam (login);
     if ( ( ! GdmAllowRoot ||
 	  ( ! GdmAllowRemoteRoot && ! local) ) &&
@@ -389,7 +549,7 @@ gdm_verify_user (GdmDisplay *d,
     }
 
     /* Check if the user's account is healthy. */
-    pamerr = pam_acct_mgmt (pamh, 0);
+    pamerr = pam_acct_mgmt (pamh, null_tok);
     switch (pamerr) {
     case PAM_SUCCESS :
 	break;
@@ -433,10 +593,8 @@ gdm_verify_user (GdmDisplay *d,
     }
 
     /* Set credentials */
-    did_setcred = TRUE;
     pamerr = pam_setcred (pamh, PAM_ESTABLISH_CRED);
     if (pamerr != PAM_SUCCESS) {
-        did_setcred = FALSE;
 	if (gdm_slave_should_complain ())
 	    gdm_error (_("Couldn't set credentials for %s"), login);
 	goto pamerr;
@@ -445,12 +603,8 @@ gdm_verify_user (GdmDisplay *d,
     credentials_set = TRUE;
 
     /* Register the session */
-    opened_session = TRUE;
     pamerr = pam_open_session (pamh, 0);
     if (pamerr != PAM_SUCCESS) {
-	    opened_session = FALSE;
-	    /* we handle this above */
-	    did_setcred = FALSE;
 	    if (gdm_slave_should_complain ())
 		    gdm_error (_("Couldn't open session for %s"), login);
 	    goto pamerr;
@@ -473,23 +627,44 @@ gdm_verify_user (GdmDisplay *d,
 	    /* I'm not sure yet if I should display this message for any other issues - heeten */
 	    if (pamerr == PAM_AUTH_ERR ||
 		pamerr == PAM_USER_UNKNOWN) {
-		    /* FIXME: Hmm, how are we sure that the login is username
-		     * and password.  That is the most common case but not
-		     * necessarily true, this message needs to be changed
-		     * to allow for such cases */
-		    auth_errmsg = 
-			    _("\nIncorrect username or password. "
-			      "Letters must be typed in the correct case. "  
-			      "Please make sure the Caps Lock key is not enabled.");
-		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
+		    gboolean is_capslock = FALSE;
+		    const char *basemsg;
+		    char *msg;
+		    char *ret;
+			    
+		    ret = gdm_slave_greeter_ctl (GDM_QUERY_CAPSLOCK, "");
+		    if ( ! ve_string_empty (ret))
+			    is_capslock = TRUE;
+		    g_free (ret);
+
+		    /* Only give this message if we actually asked for
+		       password, otherwise it would be silly to say that
+		       the password may have been wrong */
+		    if (did_we_ask_for_password) {
+			    basemsg = _("\nIncorrect username or password.  "
+					"Letters must be typed in the correct "
+					"case.");
+		    } else {
+			    basemsg = _("\nAuthentication failed.  "
+					"Letters must be typed in the correct "
+					"case.");
+		    }
+		    if (is_capslock) {
+			    msg = g_strconcat (basemsg, "  ",
+					       _("Please make sure the "
+						 "Caps Lock key is not "
+						 "enabled."),
+					       NULL);
+		    } else {
+			    msg = g_strdup (basemsg);
+		    }
+		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, msg);
+		    g_free (msg);
 	    } else {
 		    gdm_slave_greeter_ctl_no_ret (GDM_ERRDLG, _("Authentication failed"));
 	    }
     }
 
-    did_setcred = FALSE;
-    opened_session = FALSE;
-
     if (pamh != NULL) {
 	    /* Throw away the credentials */
 	    if (credentials_set)
@@ -519,10 +694,15 @@ gdm_verify_user (GdmDisplay *d,
  */
 
 gboolean
-gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display) 
+gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
+		       char **new_login) 
 {
     gint pamerr = 0;
     struct passwd *pwent;
+    const char *after_login;
+    int null_tok = 0;
+    
+    *new_login = NULL;
 
     if (login == NULL)
 	    return FALSE;
@@ -540,8 +720,27 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display)
 	    goto setup_pamerr;
     }
 
+    /* A Solaris thing: */
+#ifdef HAVE_DEFOPEN
+    if (defopen(DEFLT"/login") == 0) {
+	    char *passreq;
+	    int def_flgs;
+
+	    def_flgs = defcntl(DC_GETFLAGS, 0);
+	    TURNOFF(def_flgs, DC_CASE);
+	    (void) defcntl (DC_SETFLAGS, def_flgs);	 /* ignore case */
+
+	    if (((passreq = defread("PASSREQ=")) != NULL) &&
+		g_ascii_strcasecmp (passreq, "YES") == 0)
+		    null_tok |= PAM_DISALLOW_NULL_AUTHTOK;
+
+	    (void) defopen(NULL); /* close defaults file  */
+    }
+#endif
+
     /* Start authentication session */
-    if ((pamerr = pam_authenticate (pamh, 0)) != PAM_SUCCESS) {
+    did_we_ask_for_password = FALSE;
+    if ((pamerr = pam_authenticate (pamh, null_tok)) != PAM_SUCCESS) {
 	    if (gdm_slave_should_complain ()) {
 		    gdm_error (_("Couldn't authenticate user"));
 		    gdm_error_box (cur_gdm_disp,
@@ -551,8 +750,25 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display)
 	    goto setup_pamerr;
     }
 
+    if ((pamerr = pam_get_item (pamh, PAM_USER, (const void **)&after_login))
+	!= PAM_SUCCESS) {
+	    /* is not really an auth problem, but it will
+	       pretty much look as such, it shouldn't really
+	       happen */
+	    gdm_error (_("Couldn't authenticate user"));
+	    gdm_error_box (cur_gdm_disp,
+			   GTK_MESSAGE_ERROR,
+			   _("Authentication failed"));
+	    goto setup_pamerr;
+    }
+
+    if (after_login != NULL /* should never be */ &&
+	strcmp (after_login, login) != 0) {
+	    *new_login = g_strdup (after_login);
+    }
+
     /* Check if the user's account is healthy. */
-    pamerr = pam_acct_mgmt (pamh, 0);
+    pamerr = pam_acct_mgmt (pamh, null_tok);
     switch (pamerr) {
     case PAM_SUCCESS :
 	break;
@@ -603,21 +819,16 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display)
     }
 
     /* Set credentials */
-    did_setcred = TRUE;
     pamerr = pam_setcred (pamh, PAM_ESTABLISH_CRED);
     if (pamerr != PAM_SUCCESS) {
-	    did_setcred = FALSE;
 	    if (gdm_slave_should_complain ())
 		    gdm_error (_("Couldn't set credentials for %s"), login);
 	    goto setup_pamerr;
     }
 
     /* Register the session */
-    opened_session = TRUE;
     pamerr = pam_open_session (pamh, 0);
     if (pamerr != PAM_SUCCESS) {
-	    opened_session = FALSE;
-	    did_setcred = FALSE;
 	    /* Throw away the credentials */
 	    pam_setcred (pamh, PAM_DELETE_CRED);
 
@@ -638,9 +849,6 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display)
     return TRUE;
     
  setup_pamerr:
-
-    did_setcred = FALSE;
-    opened_session = FALSE;
     
     if (pamh != NULL)
 	    pam_end (pamh, pamerr);
@@ -673,46 +881,22 @@ gdm_verify_cleanup (GdmDisplay *d)
 
 	if (pamh != NULL) {
 		gint pamerr;
-		pam_handle_t *tmp_pamh;
-		gboolean old_opened_session;
-		gboolean old_did_setcred;
-
-		gdm_debug ("Running gdm_verify_cleanup and pamh != NULL");
-
-		gdm_sigterm_block_push ();
-		gdm_sigchld_block_push ();
-		tmp_pamh = pamh;
-		pamh = NULL;
-		old_opened_session = opened_session;
-		opened_session = FALSE;
-		old_did_setcred = did_setcred;
-		did_setcred = FALSE;
-		gdm_sigchld_block_pop ();
-		gdm_sigterm_block_pop ();
-
-		pamerr = PAM_SUCCESS;
 
 		/* Close the users session */
-		if (old_opened_session) {
-			gdm_debug ("Running pam_close_session");
-			pamerr = pam_close_session (tmp_pamh, 0);
-		}
+		pamerr = pam_close_session (pamh, 0);
 
 		/* Throw away the credentials */
-		if (old_did_setcred) {
-			gdm_debug ("Running pam_setcred with PAM_DELETE_CRED");
-			pamerr = pam_setcred (tmp_pamh, PAM_DELETE_CRED);
-		}
+		pamerr = pam_setcred (pamh, PAM_DELETE_CRED);
 
-		pam_end (tmp_pamh, pamerr);
+		if (pamh != NULL)
+			pam_end (pamh, pamerr);
+		pamh = NULL;
 
 		/* Workaround to avoid gdm messages being logged as PAM_pwdb */
 		closelog ();
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 	}
 
-	gdm_reset_limits ();
-
 	/* Clear the group setup */
 	setgid (0);
 	/* this will get rid of any suplementary groups etc... */
