@@ -12,10 +12,13 @@
 
 #include <vicious.h>
 
-#include "gdmwm.h"
+#include "greeter_configuration.h"
 #include "greeter_parser.h"
 #include "greeter_events.h"
 
+/* FIXME: hack */
+extern GreeterItemInfo *welcome_string_info;
+
 static char *file_search_path = NULL;
 
 GHashTable *item_hash = NULL;
@@ -287,11 +290,18 @@ parse_show (xmlNodePtr       node,
       xmlFree (prop);
     }
 
+  /* Note: subtype is deprecated, use type only */
   prop = xmlGetProp (node, "subtype");
   if (prop != NULL)
     {
-      g_free (info->show_subtype);
-      info->show_subtype = g_strdup (prop);
+      /* code for legacy uses of subtype only, are there any such
+       * themes out there?  The Bluecurve was the one this was made
+       * for and bluecurve is NOT using it. */
+      if (info->show_type == NULL ||
+	  strcmp (info->show_type, "system") == 0) {
+	      g_free (info->show_type);
+	      info->show_type = g_strdup (prop);
+      }
       xmlFree (prop);
     }
   
@@ -904,6 +914,7 @@ parse_translated_text (xmlNodePtr node,
  * with translation score */
 static gboolean
 parse_stock (xmlNodePtr node,
+	     GreeterItemInfo *info,
 	     char     **translated_text,
 	     gint      *translation_score,
 	     GError   **error)
@@ -926,7 +937,7 @@ parse_stock (xmlNodePtr node,
       else if (g_ascii_strcasecmp (prop, "system") == 0)
         {
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (_("S_ystem"));
+	  *translated_text = g_strdup (_("_Actions"));
 	}
       else if (g_ascii_strcasecmp (prop, "disconnect") == 0)
         {
@@ -938,6 +949,16 @@ parse_stock (xmlNodePtr node,
 	  g_free (*translated_text);
 	  *translated_text = g_strdup (_("_Quit"));
 	}
+      else if (g_ascii_strcasecmp (prop, "chooser") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_XDMCP Chooser"));
+	}
+      else if (g_ascii_strcasecmp (prop, "config") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_Configure"));
+	}
       else if (g_ascii_strcasecmp (prop, "caps-lock-warning") == 0)
         {
 	  g_free (*translated_text);
@@ -948,12 +969,13 @@ parse_stock (xmlNodePtr node,
 	  g_free (*translated_text);
 	  *translated_text = g_strdup (_("User %s will login in %d seconds"));
 	}
-      /* FIXME: this should be the welcome string but wait until
-       * we can break strings again */
       else if (g_ascii_strcasecmp (prop, "welcome-label") == 0)
         {
+	  /* FIXME: hack */
+	  welcome_string_info = info;
+
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (_("Welcome to %h"));
+	  *translated_text = g_strdup (GdmWelcome);
 	}
       /* FIXME: is this actually needed? */
       else if (g_ascii_strcasecmp (prop, "username-label") == 0)
@@ -995,7 +1017,6 @@ parse_label (xmlNodePtr        node,
 {
   xmlNodePtr child;
   int i;
-  double size_reduction = 1.0;
   char *translated_text;
   gint translation_score = 1000;
   
@@ -1028,7 +1049,7 @@ parse_label (xmlNodePtr        node,
       else if (child->type == XML_ELEMENT_NODE &&
 	       strcmp (child->name, "stock") == 0)
 	{
-	  if (!parse_stock (child, &translated_text, &translation_score, error))
+	  if (!parse_stock (child, info, &translated_text, &translation_score, error))
 	    return FALSE;
 	}
       else if (strcmp (child->name, "show") == 0)
@@ -1081,29 +1102,47 @@ parse_label (xmlNodePtr        node,
   if (info->fonts[GREETER_ITEM_STATE_NORMAL] == NULL)
     info->fonts[GREETER_ITEM_STATE_NORMAL] = pango_font_description_from_string ("Sans");
 
-  if (gdm_wm_screen.width <= 800 &&
-      gdm_wm_screen.width > 640)
-    size_reduction = PANGO_SCALE_SMALL;
-  else if (gdm_wm_screen.width <= 640)
-    size_reduction = PANGO_SCALE_X_SMALL;
-
-  if (size_reduction < 0.99)
-    {
-      for (i = 0; i < GREETER_ITEM_STATE_MAX; i++)
-        {
-          if (info->fonts[i] != NULL)
-	    {
-	      int old_size = pango_font_description_get_size (info->fonts[i]);
-	      pango_font_description_set_size (info->fonts[i], old_size * size_reduction);
-	    }
-	}
-    }
-
   info->orig_text = translated_text;
   
   return TRUE;
 }
 
+static gboolean
+parse_list (xmlNodePtr        node,
+	     GreeterItemInfo  *info,
+	     GError         **error)
+{
+  xmlNodePtr child;
+
+  child = node->children;
+  while (child)
+    {
+      if (strcmp (child->name, "pos") == 0)
+	{
+	  if (!parse_pos (child, info, error))
+	    return FALSE;
+	}
+      else if (strcmp (child->name, "show") == 0)
+	{
+	  if (!parse_show (child, info, error))
+	    return FALSE;
+	}
+      else if (strcmp (child->name, "fixed") == 0 ||
+	       strcmp (child->name, "box") == 0)
+	{
+	  g_set_error (error,
+		       GREETER_PARSER_ERROR,
+		       GREETER_PARSER_ERROR_BAD_SPEC,
+		       "List items cannot have children");
+	  return FALSE;
+	}
+    
+      child = child->next;
+    }
+
+  return TRUE;
+}
+
 static gboolean
 parse_entry (xmlNodePtr        node,
 	     GreeterItemInfo  *info,
@@ -1189,6 +1228,8 @@ parse_items (xmlNodePtr  node,
 	      item_type = GREETER_ITEM_TYPE_LABEL;
 	    else if (strcmp (type, "entry") == 0)
 	      item_type = GREETER_ITEM_TYPE_ENTRY;
+	    else if (strcmp (type, "list") == 0)
+	      item_type = GREETER_ITEM_TYPE_LIST;
 	    else
 	      {
 		g_set_error (error,
@@ -1222,6 +1263,9 @@ parse_items (xmlNodePtr  node,
 	      case GREETER_ITEM_TYPE_ENTRY:
 		res = parse_entry (child, info, error);
 		break;
+	      case GREETER_ITEM_TYPE_LIST:
+		res = parse_list (child, info, error);
+		break;
 	      default:
 		g_set_error (error,
 			     GREETER_PARSER_ERROR,
